# Copyright 2020 TestProject (https://testproject.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Class containing all the GUIDs uniquely defining the testproject() actions"""

actions = {"PAUSE_ID": "5ac5d621-7a53-45a6-94e5-a3348be3969e"}

driver_actions = {
    "SEND_KEYS_ID": "928bf14e-7a56-4371-b92a-9cb9de53f632",
    "IS_SELECTED_ID": "2c399e26-8d02-41a7-a8d4-acfeda14fb5b",
    "IS_PRESENT_ID": "a9ab4ae7-38ca-4451-a636-1b801a774503",
    "IS_VISIBLE_ID": "7314f516-588b-4dff-bf0e-7016c7e02a4b",
    "CONTAINS_TEXT_ID": "85da72d0-c5a4-4587-be33-8cb3b5fa7910",
    "IS_CLICKABLE_ID": "a59f3a20-d08d-4293-9add-b279f92e6c41",
    "CLICK_IF_VISIBLE_ID": "27fd3f33-9b63-4dc4-97e9-fcf494cbffdb",
    "IS_INVISIBLE_ID": "104373b7-868c-44ba-bbd2-e16089ad2e48",
    "TYPE_TEXT_ID": "6f94fdaa-0041-4265-b73b-ecfcbe702d66",
    "CLEAR_CONTENTS_ID": "511ddb4e-210f-4f3f-9fb9-b766bb23ee6a",
    "GET_TEXT_ID": "225caec3-3231-45d5-a3f3-c4c59b7ab385",
    "CLICK_ID": "02d572b2-d610-4700-9ad3-40076ddb9807",
    "GET_TITLE_ID": "c168339f-55c4-4c0d-91fc-cd9d4b8c7574",
}

web_actions = {
    "MOVE_MOUSE_TO_ELEMENT_ID": "631118c0-4828-42c8-aa69-0e6107d5956d",
    "NAVIGATE_FORWARD_ID": "664577f4-e1d2-4c46-b499-3934b447b478",
    "NAVIGATE_BACK_ID": "43b4408f-3fa1-4ec5-80cd-872d6dff7628",
    "REFRESH_ID": "3a06acbf-9138-4392-bca1-c1946886b65d",
    "NAVIGATE_TO_URL_ID": "49c5d8d8-4ba8-42f0-823d-702e6acfb23a",
    "GET_CURRENT_URL_ID": "56a4ad1b-b563-4322-ba29-d165090302b2",
    "SCROLL_WINDOW_ID": "a5d4eb31-51f0-4c62-8832-60c6bcd51a16",
    "SELECT_ALL_OPTIONS_BY_VALUE_ID": "45fdd921-b359-4d6a-b7c5-5f9a3612c0f2",
    "SWITCH_TO_WINDOW_ID": "3a7a12fb-66a4-47c2-aaf0-deeb7b1f748a",
    "CLOSE_WINDOW_ID": "6cc71ddb-1aaf-433c-aa94-56d944aac433",
}
