# Copyright 2020 TestProject (https://testproject.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os


class LoggingHelper:
    """Contains helper methods for setting custom logging property values"""

    @staticmethod
    def configure_logging():
        """Configures logging according to values set in dedicated environment variables"""

        logging_level = os.environ.get("TP_LOG_LEVEL", "INFO")
        logging_format = os.environ.get("TP_LOG_FORMAT", "%(asctime)s %(levelname)s %(message)s")

        logging.basicConfig(level=logging_level, format=logging_format)
