# Copyright 2020 TestProject (https://testproject.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum, unique


@unique
class ExecutionFailureType(Enum):
    """Enumeration of possible command execution failure reasons"""
    NoFailure = 0
    DriverError = 1
    MalformedExecutionJsonObject = 2
    ConditionsNotMet = 3
    RemoteResourceNotAvailable = 4
    InnerException = 5
    ValidationFailed = 6
    MissingEnvironmentCapabilities = 7
    StepCausedTestFailure = 8
    SecurityIssu = 9
